function mockRequestStack(e,t){if("https://weapp.tencent.com/jscov_driver/CollectCovTimer"==e.url)return this.origin(e);var i,s=(getCurrentPages().pop()||{}).__wxWebviewId__;console.debug(`[minium test] network stack request[${e.url}], appid[${t}], pageid[${s}]`),global.__minium_request_stack.has(s)?i=global.__minium_request_stack.get(s):(i=new Map,global.__minium_request_stack.set(s,i));var r,a=e.complete;e.complete=e=>{i.delete(r),0===i.size&&global.__minium_request_stack.delete(s),a&&a(e)};var u=this.origin(e);return r=u&&u.uniqueId||Math.random(),i.set(r,Date.now()),console.debug(`[minium test] network stack requestId[${r}], requestMap[${i.size}]`),u}
