function miniGetUserProfile(a,b){return new Promise(function(a){function c(){return a(e),e}function d(){return a(f),f}var e={errMsg:"getUserProfile:ok",userInfo:b},f={errMsg:"getUserProfile:fail auth deny"},g={};return g.允许=function(){return c()},g.拒绝=function(){return d()},g.确定=g.允许,g.取消=g.拒绝,console.warn("[minium] mock wx.getUserProfile, you can use `native.allow_get_user_info` to handle it"),global.__minium_mock_auth_modal(g)})}