function createContext(a,b){if(global.__minium.$_createMediaContext_$s||(global.__minium.$_createMediaContext_$s={}),!global.__minium.$_createMediaContext_$s["".concat(b,"_").concat(a)]){global.__minium.$_createMediaContext_$s["".concat(b,"_").concat(a)]=wx.$_createMediaContext_$(a);var c=getCurrentPages().pop(),d=c.onUnload;c.onUnload=function(){delete global.__minium.$_createMediaContext_$s["".concat(b,"_").concat(a)];for(var c=arguments.length,e=Array(c),f=0;f<c;f++)e[f]=arguments[f];d.call.apply(d,[this].concat(e))}.bind(c)}}