#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
"""
Author:         yopofeng
Filename:       utils.py
Create time:    2022/9/27 17:12
Description:

"""
from .utils import ProcessSafeEventLoop

event_loop = ProcessSafeEventLoop()
