function miniSubscribeMessage(e){let s=e.tmplIds;return new Promise((e=>{let r={"允许":function(){let r={errMsg:"requestSubscribeMessage:ok"};for(let e=0;e<s.length;e++)r[s[e]]="accept";return e(r),r},"取消":function(){let r={errMsg:"requestSubscribeMessage:ok"};for(let e=0;e<s.length;e++)r[s[e]]="reject";return e(r),r}};return console.warn("[minium] mock wx.requestSubscribeMessage, you can use `native.allow_send_subscribe_message` to handle it"),global.__minium_mock_auth_modal(r)}))}
