function miniGetLocation(a,b,c){return new Promise(function(a){function d(){var b={};b.确定=function(){return a(f),f},b[f.name]=function(){return a(f),f};var c=function _loop(c){b[c]=function(){return a(i[c]),i[c]}};for(var d in i)c(d);b.取消=function(){return a(g),g},global.__minium_mock_map_modal(b)}function e(){return a(h),h}var f={errMsg:"chooseLocation:ok",name:"\u817E\u8BAF\u5FAE\u4FE1\u603B\u90E8",address:"\u5E7F\u4E1C\u7701\u5E7F\u5DDE\u5E02",latitude:23.12463,longitude:113.36199},g={errMsg:"chooseLocation:fail cancel"},h={errMsg:"chooseLocation:fail auth deny"},i={};for(var j in f)void 0!==b[j]&&(f[j]=b[j]);for(var k in c)i[k]=Object.assign({},f,c[k]);if(!0===global.__minium_auth_setting["scope.userLocation"])return d();if(!1===global.__minium_auth_setting["scope.userLocation"])return e();var m={};return m.允许=function(){return global.__minium_auth_setting["scope.userLocation"]=!0,d()},m.拒绝=function(){return global.__minium_auth_setting["scope.userLocation"]=!1,e()},m.确定=m.允许,m.取消=m.拒绝,console.warn("[minium] mock wx.chooseLocation, you can use `native.allow_get_location` to handle it"),global.__minium_mock_auth_modal(m)})}