function miniGetLocation(n,o,t){return new Promise((n=>{let i={errMsg:"chooseLocation:ok",name:"腾讯微信总部",address:"广东省广州市",latitude:23.12463,longitude:113.36199},e={errMsg:"chooseLocation:fail cancel"},u={errMsg:"chooseLocation:fail auth deny"},a={};for(let n in i)void 0!==o[n]&&(i[n]=o[n]);for(let n in t)a[n]=Object.assign({},i,t[n]);function r(){let o={"确定":function(){return n(i),i}};o[i.name]=function(){return n(i),i};for(let t in a)o[t]=function(){return n(a[t]),a[t]};o["取消"]=function(){return n(e),e},global.__minium_mock_map_modal(o)}function c(){return n(u),u}if(!0===global.__minium_auth_setting["scope.userLocation"])return r();if(!1===global.__minium_auth_setting["scope.userLocation"])return c();let l={"允许":function(){return global.__minium_auth_setting["scope.userLocation"]=!0,r()},"拒绝":function(){return global.__minium_auth_setting["scope.userLocation"]=!1,c()}};return l["确定"]=l["允许"],l["取消"]=l["拒绝"],console.warn("[minium] mock wx.chooseLocation, you can use `native.allow_get_location` to handle it"),global.__minium_mock_auth_modal(l)}))}
