function miniGetWeRunData(n,t,e){return new Promise((n=>{let u={errMsg:"getWeRunData:ok",encryptedData:t,iv:e},a={errMsg:"getWeRunData:fail auth deny"};function i(){return n(u),u}function r(){return n(a),a}if(!0===global.__minium_auth_setting["scope.werun"])return i();if(!1===global.__minium_auth_setting["scope.werun"])return r();let _={"允许":function(){return global.__minium_auth_setting["scope.werun"]=!0,i()},"拒绝":function(){return global.__minium_auth_setting["scope.werun"]=!1,r()}};return _["确定"]=_["允许"],_["取消"]=_["拒绝"],console.warn("[minium] mock wx.getWeRunData, you can use `native.allow_get_we_run_data` to handle it"),global.__minium_mock_auth_modal(_)}))}
