function miniumRequestStack(){if(!global.__minium_request_stack){global.__minium_request_stack=new Map,global.__minium_wait_util=function(){var a=0<arguments.length&&void 0!==arguments[0]?arguments[0]:0,b=1<arguments.length&&void 0!==arguments[1]?arguments[1]:20,c=(getCurrentPages().pop()||{}).__wxWebviewId__;return new Promise(function(d){var e=10*b,f=setInterval(function(){e--;var b=global.__minium_request_stack.has(c)?global.__minium_request_stack.get(c).size:0;console.debug("[minitest] current stack size: ",b),b<=a?(clearInterval(f),d(!0)):0>e&&(clearInterval(f),d(!1))},100)})};var a=wx.request;return Object.defineProperty(wx,"request",{configurable:!0,get:function get(){return function(b){if("https://weapp.tencent.com/jscov_driver/CollectCovTimer"==b.url)return a(b);var c,d=(getCurrentPages().pop()||{}).__wxWebviewId__;global.__minium_request_stack.has(d)?c=global.__minium_request_stack.get(d):(c=new Map,global.__minium_request_stack.set(d,c));var e,f=b.complete;b.complete=function(a){c["delete"](e),0===c.size&&global.__minium_request_stack["delete"](d),f&&f(a)};var g=a(b);return e=g&&g.uniqueId||Math.random(),c.set(e,Date.now()),g}}}),!0}return!1}