function miniGetLocation(t,e){return new Promise((i=>{let o;o="gcj02"===t.type?{speed:-1,steps:0,errMsg:"getLocation:ok",provider:"gps",buildingId:"",longitude:113.32489013671875,latitude:23.099933624267578,verticalAccuracy:10,floorName:"1000",accuracy:65,direction:-1,horizontalAccuracy:65}:{speed:-1,steps:0,errMsg:"getLocation:ok",provider:"gps",buildingId:"",longitude:113.31950378417969,latitude:23.102487564086914,verticalAccuracy:10,floorName:"1000",accuracy:65,direction:-1,horizontalAccuracy:65},t.altitude&&(o.altitude=23.102540969848633);for(let t in o)void 0!==e[t]&&(o[t]=e[t]);let n={errMsg:"getLocation:fail auth deny"};function r(){return i(o),o}function a(){return i(n),n}if(!0===global.__minium_auth_setting["scope.userLocation"])return r();if(!1===global.__minium_auth_setting["scope.userLocation"])return a();let c={"允许":function(){return global.__minium_auth_setting["scope.userLocation"]=!0,r()},"拒绝":function(){return global.__minium_auth_setting["scope.userLocation"]=!1,a()}};return c["确定"]=c["允许"],c["取消"]=c["拒绝"],console.warn("[minium] mock wx.getLocation, you can use `native.allow_get_location` to handle it"),global.__minium_mock_auth_modal(c)}))}
