function enableNetworkPanel(){let e=wx.request;if(!global.__minium_request_hooked){Object.defineProperty(global,"__minium_request_hooked",{value:!0,writable:!1});const t=Object.getOwnPropertyDescriptor(wx,"request");if(t&&!1===t.configurable)return void console.warn("[minitest] Cannot redefine property: request");let r=function(e){let t=5381,r=e.length;for(;r;)t=33*t^e.charCodeAt(--r);return t>>>0},i=new Map,n=new Map;Object.defineProperty(wx,"request",{configurable:!0,get:()=>function(t){if(!global||!global.__minium_get_mini_network_id)return e(t);if("https://weapp.tencent.com/jscov_driver/CollectCovTimer"==t.url)return e(t);let l,o=global.__minium_get_mini_network_id(),s=Object.assign({},t);delete s.success,delete s.fail,delete s.complete;let u=Date.now();try{let e=JSON.stringify(s);l=r(e),i.has(l)&&i.get(l)>u&&(e=""),i.set(l,u+21e3),mini_send_request(o,e,u,l)}catch(e){mini_send_request(o,JSON.stringify({url:s.url,err:e.toString()}),u,0)}let _=t.complete;return t.complete=e=>{let i=Object.assign({},e);delete i.profile;let l=JSON.stringify(i),s=r(l),u=Date.now();n.has(s)&&n.get(s)>u&&(l=""),n.set(s,u+21e3),mini_request_callback(o,l,u,s,t.__miniumMocked),_&&_(e)},e(t)}})}}
