function miniActionSheet(e){return new Promise((n=>{let t={},o=e.itemList;for(let e=0;e<o.length;e++)t[o[e]]=function(){let t={errMsg:"showActionSheet:ok",tapIndex:e};return n(t),t};return t["取消"]=function(){let e={errMsg:"showActionSheet:fail cancel"};return n(e),e},console.warn("[minium] mock wx.showActionSheet, you can use `native.handle_action_sheet` to handle it"),global.__minium_mock_native_modal(t)}))}
