function miniShowModal(a){var b=a.confirmText||"\u786E\u5B9A",c=a.cancelText||"\u53D6\u6D88";return new Promise(function(a){var d={};return d[b]=function(){var b={errMsg:"showModal:ok",confirm:!0,cancel:!1};return a(b),b},d[c]=function(){var b={errMsg:"showModal:ok",confirm:!1,cancel:!0};return a(b),b},console.warn("[minium] mock wx.showModal, you can use `native.handle_modal` to handle it"),global.__minium_mock_native_modal(d)})}