function editEditorText(){return global.__minium.edRes="",new Promise((e=>{wx.createSelectorQuery().select("#%(editorid)s").context((function(t){t.context.insertText({text:"%(text)s",success(t){global.__minium.edRes=t.errMsg,e(t.errMsg)},fail(t){global.__minium.edRes=t.errMsg,e(t.errMsg)}})})).exec()}))}
