function cleanCloudCallMockRule(a){global.__minium_cloud_call_mocked&&global["__minium_".concat(a,"_mock_rule")]&&global["__minium_".concat(a,"_mock_rule")].splice(0,global["__minium_".concat(a,"_mock_rule")].length)}