function miniActionSheet(a){return new Promise(function(b){for(var c={},d=a.itemList,e=function _loop(a){c[d[a]]=function(){var c={errMsg:"showActionSheet:ok",tapIndex:a};return b(c),c}},f=0;f<d.length;f++)e(f);return c.取消=function(){var a={errMsg:"showActionSheet:fail cancel"};return b(a),a},console.warn("[minium] mock wx.showActionSheet, you can use `native.handle_action_sheet` to handle it"),global.__minium_mock_native_modal(c)})}