function ideMockAuth(t){let e=t.scope;return new Promise((t=>{wx.getSetting({success(i){if(!0===i.authSetting[e])t({errMsg:"authorize:ok"});else if(!0===i.authSetting[e])t({errMsg:"authorize:fail auth deny"});else{let i={"允许":function(){global.__minium_auth_setting[e]=!0,t({errMsg:"authorize:ok"})},"拒绝":function(){global.__minium_auth_setting[e]=!1,t({errMsg:"authorize:fail auth deny"})}};i["不允许"]=i["拒绝"],global.__minium_mock_auth_modal(i)}}})}))}
