function miniSubscribeMessage(a){var b=a.tmplIds;return new Promise(function(a){var c={};return c.允许=function(){for(var c={errMsg:"requestSubscribeMessage:ok"},d=0;d<b.length;d++)c[b[d]]="accept";return a(c),c},c["\u53D6\u6D88"]=function(){for(var c={errMsg:"requestSubscribeMessage:ok"},d=0;d<b.length;d++)c[b[d]]="reject";return a(c),c},console.warn("[minium] mock wx.requestSubscribeMessage, you can use `native.allow_send_subscribe_message` to handle it"),global.__minium_mock_auth_modal(c)})}