function miniGetUserProfile(e,r){return new Promise((e=>{let n={errMsg:"getUserProfile:ok",userInfo:r},o={errMsg:"getUserProfile:fail auth deny"};let t={};return t["允许"]=function(){return e(n),n},t["拒绝"]=function(){return e(o),o},t["确定"]=t["允许"],t["取消"]=t["拒绝"],console.warn("[minium] mock wx.getUserProfile, you can use `native.allow_get_user_info` to handle it"),global.__minium_mock_auth_modal(t)}))}
