#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Created by xiazeng on 2018/6/26

LauncherUI = "com.tencent.mm.ui.LauncherUI"

APP_LAUNCHER = "com.tencent.mm.plugin.appbrand.ui.AppBrandLauncherUI"
LUGGAGE_LAUNCHER = "com.tencent.wmpf.dev.MainActivity"
ALBUM = "com.tencent.mm.plugin.gallery.ui.AlbumPreviewUI"

BindContactUI = "com.tencent.mm.plugin.account.bind.ui.BindMContactUI"

SelectConversationUI = "com.tencent.mm.ui.transmit.SelectConversationUI"
SoSoProxyUI = "com.tencent.mm.plugin.location_soso.SoSoProxyUI"
AppInstallerUI = "com.tencent.mm.sandbox.updater.AppInstallerUI"


# 小程序预览图片
GestureGalleryUI = "com.tencent.mm.plugin.subapp.ui.gallery.GestureGalleryUI"
PAY_WalletPayUI = "com.tencent.mm.plugin.wallet.pay.ui.WalletPayUI"

# 小程序的界面， 有可能出现 com.tencent.mm.plugin.appbrand.ui.AppBrandUI1
AppBrandUI = "com.tencent.mm.plugin.appbrand.ui.AppBrandUI"

# 授权页面
AppBrandAuthorizeUI = "com.tencent.mm.plugin.appbrand.ui.AppBrandAuthorizeUI"
AppBrandAuthorizeUI_1 = "com.tencent.mm.plugin.appbrand.ui.AppBrandAuthorizeUI1"
AppBrandAuthorizeUI_2 = "com.tencent.mm.plugin.appbrand.ui.AppBrandAuthorizeUI2"
#地理位置页面
LocationUI = "com.tencent.mm.plugin.location_soso.SoSoProxyUI"
# 登录界面
LoginPasswordUI = "com.tencent.mm.plugin.account.ui.LoginPasswordUI"


AppBrandOpenWeRunSettingUI = "com.tencent.mm.plugin.appbrand.ui.AppBrandOpenWeRunSettingUI"

