function getUni(){if(!wx.__webpack_require_UNI_MP_PLUGIN__)return"not uni app";if("undefined"==typeof global||!global.webpackJsonp)return"global.webpackJsonp not exists";const e=global.webpackJsonp;for(let n=0;n<e.length;n++){let t,o,[r,a,i]=e[n];r="string"==typeof r?[r]:r;for(let e=0;e<r.length;e++)if("common/vendor"===r[e]){t=a;break}if(!t)return"can't find `common/vendor`";for(let e in t){const n=wx.__webpack_require_UNI_MP_PLUGIN__(e).default;if(n&&(n.createApp&&n.createPage&&n.request)){o=n;break}}return o?void(global.uni=o):"can't find uni"}}
