function hookWxMethod(){var e;wx.$_origin_$?e=wx.$_origin_$:(e=wx.$_method_$,Object.defineProperty(wx,"$_origin_$",{configurable:!0,get:()=>e})),Object.defineProperty(wx,"$_method_$",{configurable:!0,get:()=>function(...t){var n,$=t[0],_=!1;if("$_method_$".endsWith("Sync")||$&&($.success||$.fail||$.complete)||(_=!0),"undefined"!=typeof $_before_$&&$_before_$(...t),"undefined"!=typeof $_callback_$)if(_)n=e(...t).then((function(e){return $_callback_$(e),e})).catch((function(e){throw $_callback_$(e),e}));else{if($){var o=$.complete;$.complete=function(e){$_callback_$(e),o&&o(e)}}n=e(...t)}else n=e(...t);return"undefined"!=typeof $_after_$&&$_after_$(n),n}})}
