function miniGetWeRunData(a,b,c){return new Promise(function(a){function d(){return a(f),f}function e(){return a(g),g}var f={errMsg:"getWeRunData:ok",encryptedData:b,iv:c},g={errMsg:"getWeRunData:fail auth deny"};if(!0===global.__minium_auth_setting["scope.werun"])return d();if(!1===global.__minium_auth_setting["scope.werun"])return e();var h={};return h.允许=function(){return global.__minium_auth_setting["scope.werun"]=!0,d()},h.拒绝=function(){return global.__minium_auth_setting["scope.werun"]=!1,e()},h.确定=h.允许,h.取消=h.拒绝,console.warn("[minium] mock wx.getWeRunData, you can use `native.allow_get_we_run_data` to handle it"),global.__minium_mock_auth_modal(h)})}