function addNetworkMockRule(_){return global["__minium_%(interface)s_network_mock_rule"]||(global["__minium_%(interface)s_network_mock_rule"]=[]),global["__minium_%(interface)s_network_mock_rule"].push(_),!!global["__minium_%(interface)s_network_mocked"]}
