function hookWxMethod(){var a;wx.$_origin_$?a=wx.$_origin_$:(a=wx.$_method_$,Object.defineProperty(wx,"$_origin_$",{configurable:!0,get:function get(){return a}})),Object.defineProperty(wx,"$_method_$",{configurable:!0,get:function get(){return function(){var b=0>=arguments.length?void 0:arguments[0],c=!1;"$_method_$".endsWith("Sync")||b&&(b.success||b.fail||b.complete)||(c=!0),"undefined"!=typeof $_before_$&&$_before_$.apply(void 0,arguments);var d;if("undefined"==typeof $_callback_$)d=a.apply(void 0,arguments);else if(c)d=a.apply(void 0,arguments).then(function(a){return $_callback_$(a),a})["catch"](function(a){throw $_callback_$(a),a});else{if(b){var e=b.complete;b.complete=function(a){$_callback_$(a),e&&e(a)}}d=a.apply(void 0,arguments)}return"undefined"!=typeof $_after_$&&$_after_$(d),d}}})}