function mockCloudCall(){if(!global.__minium_cloud_call_mocked){var a=function(b,c){if(global.babelHelpers["typeof"](b)!==global.babelHelpers["typeof"](c))return!1;switch(global.babelHelpers["typeof"](b)){case"string":var d=new RegExp(b);return!!d.exec(c);case"number":return b===c;case"object":if(b instanceof Array){if(b.length!=c.length)return!1;for(var e=0;e<b.length;e++)if(!a(b[e],c[e]))return!1;return!0}for(var f in b)if("success"!=f&&"fail"!=f&&"complete"!=f&&"_miniMockType"!=f){if(void 0===c[f])return!1;if(!a(b[f],c[f]))return!1}return!0}return!1},b=function(b){return function(d){if(!global["__minium_".concat(b,"_mock_rule")])return c[b].call(this,d);for(var e,f=0;f<global["__minium_".concat(b,"_mock_rule")].length;f++)if(e=global["__minium_".concat(b,"_mock_rule")][f],a(e,d)){if(1===e._miniMockType){var g=global["__minium_".concat(b,"_mock_rule")].splice(f,1),h=global.babelHelpers.slicedToArray(g,1);e=h[0]}if(console.log("[minium] ".concat(b," rule match"),e),e.success)return Promise.resolve(e.success);if(e.fail)return Promise.reject(e.fail);if(1===e._miniMockType)return}return c[b].call(this,d)}};Object.defineProperty(global,"__minium_cloud_call_mocked",{value:!0,writable:!1});var c={},d=["callContainer","callFunction"];if(d.forEach(function(a){c[a]=wx.cloud[a]}),"undefined"!=typeof Proxy){var e=new Proxy(wx.cloud,{get:function get(a,c){return-1===d.indexOf(c)?a[c]:b(c)}});wx.cloud=e,Object.defineProperty(wx,"cloud",{writable:!0,value:e})}else d.forEach(function(a){var c=b(a);Object.defineProperty(wx.cloud,a,{writable:!0,value:c}),wx.cloud[a]=c})}}