function miniShowModal(n){let o=n.confirmText||"确定",e=n.cancelText||"取消";return new Promise((n=>{let l={};return l[o]=function(){let o={errMsg:"showModal:ok",confirm:!0,cancel:!1};return n(o),o},l[e]=function(){let o={errMsg:"showModal:ok",confirm:!1,cancel:!0};return n(o),o},console.warn("[minium] mock wx.showModal, you can use `native.handle_modal` to handle it"),global.__minium_mock_native_modal(l)}))}
