function enableNetworkPanel(){var a=wx.request;if(!global.__minium_request_hooked){Object.defineProperty(global,"__minium_request_hooked",{value:!0,writable:!1});var b=Object.getOwnPropertyDescriptor(wx,"request");if(b&&!1===b.configurable)return void console.warn("[minitest] Cannot redefine property: request");var c=function(a){for(var b=5381,c=a.length;c;)b=33*b^a.charCodeAt(--c);return b>>>0},d=new Map,e=new Map;Object.defineProperty(wx,"request",{configurable:!0,get:function get(){return function(b){if(!global||!global.__minium_get_mini_network_id)return a(b);if("https://weapp.tencent.com/jscov_driver/CollectCovTimer"==b.url)return a(b);var f=global.__minium_get_mini_network_id(),g=Object.assign({},b);delete g.success,delete g.fail,delete g.complete;var h,i=Date.now();try{var j=JSON.stringify(g);h=c(j),d.has(h)&&d.get(h)>i&&(j=""),d.set(h,i+21e3),mini_send_request(f,j,i,h)}catch(a){mini_send_request(f,JSON.stringify({url:g.url,err:a.toString()}),i,0)}var k=b.complete;return b.complete=function(a){var d=Object.assign({},a);delete d.profile;var g=JSON.stringify(d),h=c(g),i=Date.now();e.has(h)&&e.get(h)>i&&(g=""),e.set(h,i+21e3),mini_request_callback(f,g,i,h,b.__miniumMocked),k&&k(a)},a(b)}}})}}