function miniGetLocation(a,b){return new Promise(function(c){function d(){return c(f),f}function e(){return c(h),h}var f;for(var g in f="gcj02"===a.type?{speed:-1,steps:0,errMsg:"getLocation:ok",provider:"gps",buildingId:"",longitude:113.32489013671875,latitude:23.099933624267578,verticalAccuracy:10,floorName:"1000",accuracy:65,direction:-1,horizontalAccuracy:65}:{speed:-1,steps:0,errMsg:"getLocation:ok",provider:"gps",buildingId:"",longitude:113.31950378417969,latitude:23.102487564086914,verticalAccuracy:10,floorName:"1000",accuracy:65,direction:-1,horizontalAccuracy:65},a.altitude&&(f.altitude=23.102540969848633),f)void 0!==b[g]&&(f[g]=b[g]);var h={errMsg:"getLocation:fail auth deny"};if(!0===global.__minium_auth_setting["scope.userLocation"])return d();if(!1===global.__minium_auth_setting["scope.userLocation"])return e();var i={};return i.允许=function(){return global.__minium_auth_setting["scope.userLocation"]=!0,d()},i.拒绝=function(){return global.__minium_auth_setting["scope.userLocation"]=!1,e()},i.确定=i.允许,i.取消=i.拒绝,console.warn("[minium] mock wx.getLocation, you can use `native.allow_get_location` to handle it"),global.__minium_mock_auth_modal(i)})}