function getUni(){if(!wx.__webpack_require_UNI_MP_PLUGIN__)return"not uni app";if("undefined"==typeof global||!global.webpackJsonp)return"global.webpackJsonp not exists";for(var a=global.webpackJsonp,b=0;b<a.length;b++){var c=global.babelHelpers.slicedToArray(a[b],3),d=c[0],e=c[1],f=c[2],g=void 0;d="string"==typeof d?[d]:d;for(var h=0;h<d.length;h++)if("common/vendor"===d[h]){g=e;break}if(!g)return"can't find `common/vendor`";var k=void 0;for(var l in g){var m=wx.__webpack_require_UNI_MP_PLUGIN__(l)["default"];if(m&&m.createApp&&m.createPage&&m.request){k=m;break}}return k?void(global.uni=k):"can't find uni"}}