function createContext(e,t){if(global.__minium.$_createMediaContext_$s||(global.__minium.$_createMediaContext_$s={}),!global.__minium.$_createMediaContext_$s[`${t}_${e}`]){global.__minium.$_createMediaContext_$s[`${t}_${e}`]=wx.$_createMediaContext_$(e);var _=getCurrentPages().pop(),a=_.onUnload;_.onUnload=function(..._){delete global.__minium.$_createMediaContext_$s[`${t}_${e}`],a.call(this,..._)}.bind(_)}}
