function ideMockAuth(a){var b=a.scope;return new Promise(function(a){wx.getSetting({success:function success(c){if(!0===c.authSetting[b])a({errMsg:"authorize:ok"});else if(!0===c.authSetting[b])a({errMsg:"authorize:fail auth deny"});else{var d={};d.允许=function(){global.__minium_auth_setting[b]=!0,a({errMsg:"authorize:ok"})},d.拒绝=function(){global.__minium_auth_setting[b]=!1,a({errMsg:"authorize:fail auth deny"})},d.不允许=d.拒绝,global.__minium_mock_auth_modal(d)}}})})}