function mockRequestStack(a,b){if("https://weapp.tencent.com/jscov_driver/CollectCovTimer"==a.url)return this.origin(a);var c=(getCurrentPages().pop()||{}).__wxWebviewId__;console.debug("[minium test] network stack request[".concat(a.url,"], appid[").concat(b,"], pageid[").concat(c,"]"));var d;global.__minium_request_stack.has(c)?d=global.__minium_request_stack.get(c):(d=new Map,global.__minium_request_stack.set(c,d));var e,f=a.complete;a.complete=function(a){d["delete"](e),0===d.size&&global.__minium_request_stack["delete"](c),f&&f(a)};var g=this.origin(a);return e=g&&g.uniqueId||Math.random(),d.set(e,Date.now()),console.debug("[minium test] network stack requestId[".concat(e,"], requestMap[").concat(d.size,"]")),g}