pre[data-v-015e70d9]{margin:0;white-space:pre-wrap;word-wrap:break-word}.log-box[data-v-015e70d9]{padding:1em 0 0 .5em;background-color:#3c3f41}.log-debug[data-v-015e70d9]{color:#c1c6cf}.log-info[data-v-015e70d9]{color:#67c23a}.log-warning[data-v-015e70d9]{color:#e6a23c}.log-error[data-v-015e70d9]{color:#f56c6c}.el-table .warning-row{background:#e6a23c}.el-table .success-row{background:#67c23a}.el-table .error-row{background:#f56c6c}